module Main where

import Graphics.Gloss
import System.Random


main :: IO ()
main = do
	simulate (InWindow "Paying Attention" (600, 800) (5, 5))
                (greyN 0.98)
		60
		(0.0, [], 0)
		frame
		u

u _ _ (t, ds, gn) = (t+1, uds t ds gn, gn)

data Doodle = Doodle (Float -> [Int] -> Picture) Float Float Float [Int]

uds :: Float -> [Doodle] -> Int -> [Doodle]
uds t ds gn | mod' t 100 == 0 = Doodle (doodleFunctions!!(head (randomRs (0, (length doodleFunctions)-1) g))) t (head (randomRs ((-240),100) g)) (head (randomRs ((-400),0) g)) (tail (randomRs (0,100) g)) : ds
		    | otherwise       = filter (\(Doodle f st x y s) -> t < st+1600) ds
				where g = mkStdGen (round t) 

frame :: (Float, [Doodle], Int) -> Picture
frame (t, ds, gn)
 = thick $ Pictures	
	[ margin t,
	  Pictures [Translate x (y + (t-st)/2) $ Rotate (cos(st)*20) $ Scale (cos(st*2.5)/2+1.5) (cos(st*2.5+0.1)/2+1.5) $ Color ([black, blue]!!(mod (head s) 2)) $ Pictures [f (tt/2) s | tt <- [0..(t-st)/2 - (mod' ((t-st)/2) 1)]] | Doodle f st x y s <- ds], 
	  holePunch t ]

thick :: Picture -> Picture
thick p = Pictures [Translate 1 0 p, Translate 0 (-1) p, p]

margin :: Float -> Picture
margin t = Pictures ([ (\y -> Color (greyN 0.5) $ line [(-300, y), (300, y)]) (n*80 + mod' (t/2) 80) | n <- [-5..5]]
	++ [ Color red $ line [(100,-400), (100,400)] ])

holePunch :: Float -> Picture
holePunch t = Pictures ([ (\y -> Color (greyN 0.05) $ Translate (-250) y $ ThickCircle 7 15) (n*125 + mod' (t/2) 125) | n <- [-5..4]])

mod' :: Float -> Float -> Float
mod' n d = n - (fromIntegral (floor (n/d))) * d

--doodles

type DoodleFunction = Float -> [Int] -> Picture

doodleFunctions = [lissajous, star, star, flipd star, flower, flower, heart, \f t -> Color magenta $ heart f t, twist, spiral, flipd twist, flipd spiral]

spiral :: DoodleFunction
spiral = connect 50 f 
	where f t s = (sin(t/y + x)*t*1.5, cos(t/y + x)*t)
			where 
				x = fromIntegral (head s)
				y = fromIntegral (s!!2)/10 + 1

twist :: DoodleFunction
twist = connect 60 f
	where f t s = (40*(-sin(t*y) * cos(t*y) + t/x) - 50, (30+(fromIntegral (s!!2))/50)*cos(t*y))
		  	where
		  		x = (fromIntegral (head s)) / 10 + 10
		  		y = ((fromIntegral (s!!3))/25 + 2)/10

heart :: DoodleFunction
heart = connect 30 f
	where f t s = (20 * (sqrt 2) * (sin tt) ^ 3, 20 * (-(cos tt) ^ 3 - (cos tt) ^ 2 + (cos tt) * 2) + 4 * sin((t/10)*1.34 + (fromIntegral (head s))))
			where
				tt = t/5

flower :: DoodleFunction
flower = connect 65 f
	where f t s = (20*sin(tt) + x*sin(tt*y), 20*cos(tt) + x*cos(tt*y))
			where
				x = (fromIntegral (head s))/10 + 10
				y = (fromIntegral (head s))/100 * 4 + 3
				tt = t/10

star :: DoodleFunction
star = connect 33 f
	where f t s = (20*sin(tt*2) + x*sin(-tt*3), 20*cos(tt*2) + x*cos(-tt*3) - t/10)
			where
				x = (fromIntegral (head s))/10 + 7
				tt = t/5

lissajous :: DoodleFunction
lissajous = connect 60 f
	where f t s = (14*sin(tt*x), 14*cos(tt*y))
			where
				x = mod' (fromIntegral (head s)) 10
				y = mod' (fromIntegral (s!!5)) 10
				tt = t/10

--helpers for doodles

flipd f t s = Scale (-1) 1 $ f t s

connect finish f t s | t < finish = line [f t s, f (t+0.5) s]
			  | otherwise = Pictures []



